# Worst case distance

from definitions import *
from pyopus.design.wcd import WorstCaseDistance
# If MPI is imported an application not using MPI will behave correctly
# (i.e. only slot 0 will run the program) even when started with mpirun/mpiexec
from pyopus.parallel.mpi import MPI
from pyopus.parallel.cooperative import cOS


if __name__=='__main__':
	atDesign={
		# Result of design across corners
		'c_out':    6.4025057781114802e-12,
		'diff_l':   3.4247816773735976e-07,
		'diff_w':   1.6738092946223754e-05,
		'load_l':   2.3668072815514710e-06,
		'load_w':   5.3240803355989389e-05,
		'mirr_l':   1.0157035640188838e-06,
		'mirr_ld':  1.6182520447950098e-06,
		'mirr_w':   5.8107861163445226e-05,
		'mirr_wd':  6.1242813846216839e-05,
		'mirr_wo':  6.8277846319352392e-05,
		'out_l':    4.1088626829554061e-07,
		'out_w':    8.1812469568552345e-05,
		'r_out':    5.1647907484619873e+02,
	}
	
	# Create corner with the Monte Carlo model
	corners={
		'mc': {
			'params': {}, 
			'modules': [ 'mc' ]
		}
	}
		
	# Worst case performances (skip area)
	wcList=list(measures.keys())
	wcList.sort()
	wcList.remove('area')
	
	# Prepare parallel environment
	cOS.setVM(MPI(mirrorMap={'*':'.'}))
	
	# 3-sigma worst case
	wc=WorstCaseDistance(
		heads, analyses, measures, corners, statParams, opParams, variables=variables, 
		fixedParams=atDesign, 
		stepTol=0.01,  # Try 0.0001
		sigmaBox=10, 
		debug=2, 
		spawnerLevel=0
	)
	wcresults, analysisCount = wc(wcList)
	# wcresults, analysisCount = wc(['vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr', 'vgs_drv'])
	# wcresults, analysisCount = wc(['cmrr'])
	# wcresults, analysisCount = wc(['ugbw'])
	# wcresults, analysisCount = wc(['vgs_drv'])
	
	print(wc.formatResults())
	print(wc.analysisCount)
	
	# Finalize cOS parallel environment
	cOS.finalize()

# Results 
#       cmrr >     6.00000e+01 wc=    5.99891e+01 wcd=    1.91679e+00 neval= 277 1 OK
#       gain >     6.00000e+01 wc=    6.28073e+01 wcd=    1.73205e+01 neval= 170 2 OUTSIDE+
#       isup <     1.00000e-03 wc=    3.52207e-04 wcd=    2.23607e+01 neval= 146 1 SENS+
#     out_op >    -1.25000e-02 wc=   -1.25000e-02 wcd=    4.06236e+00 neval= 158 1 OK
#     out_op <     1.25000e-02 wc=    1.25002e-02 wcd=    1.52293e+00 neval= 119 1 OK
#   overshdn <     1.50000e-01 wc=    1.49979e-01 wcd=    8.97988e+00 neval= 549 2 OK
#   overshup <     1.50000e-01 wc=    1.49939e-01 wcd=    6.90624e+00 neval= 230 2 OK
#   ph_slope <     0.00000e+00 wc=   -0.00000e+00 wcd=    0.00000e+00 neval= 137 1 SENS0
#         pm >     5.00000e+01 wc=    5.49886e+01 wcd=    1.73205e+01 neval= 141 1 OUTSIDE+
#   psrr_vdd >     6.00000e+01 wc=    6.00000e+01 wcd=    9.57092e+00 neval= 254 2 OK
#   psrr_vss >     6.00000e+01 wc=    5.99952e+01 wcd=    1.17204e+01 neval=1444 4 OK
#     slewdn >     2.00000e+06 wc=    6.47336e+06 wcd=    2.98603e+01 neval= 806 1 SENS+
#     slewup >     2.00000e+06 wc=    2.43372e+06 wcd=    1.75397e+01 neval= 328 2 OUTSIDE+
#      swing >     1.00000e+00 wc=    1.17516e+00 wcd=    1.82080e+01 neval= 160 1 SENS+
#     tsetdn <     1.00000e-06 wc=    2.45628e-07 wcd=    2.44053e+01 neval= 180 1 SENS+
#     tsetup <     1.00000e-06 wc=    8.19844e-07 wcd=    2.32487e+01 neval= 159 1 SENS+
#       ugbw >     1.00000e+07 wc=    1.00042e+07 wcd=    9.46010e+00 neval= 138 1 OK
# vds_drv[0] >     0.00000e+00 wc=    7.15038e-06 wcd=    1.47031e+01 neval= 129 1 OK
# vds_drv[1] >     0.00000e+00 wc=   -1.75628e-02 wcd=    1.51335e+01 neval= 203 1 OK
# vds_drv[2] >     0.00000e+00 wc=    6.12508e-02 wcd=    1.00000e+01 neval= 133 1 OUTSIDE+
# vds_drv[3] >     0.00000e+00 wc=   -3.55896e-06 wcd=    5.04795e+00 neval= 121 1 OK
# vds_drv[4] >     0.00000e+00 wc=    6.08470e-01 wcd=    2.64810e+01 neval= 261 1 SENS+
# vds_drv[5] >     0.00000e+00 wc=    8.90767e-02 wcd=    1.41421e+01 neval= 125 1 OUTSIDE+
# vds_drv[6] >     0.00000e+00 wc=   -5.14184e-09 wcd=    8.62122e+00 neval= 115 1 OK
# vds_drv[7] >     0.00000e+00 wc=    6.00976e-01 wcd=    2.31280e+01 neval= 207 1 SENS+
# vgs_drv[0] >     0.00000e+00 wc=   -2.04971e-10 wcd=    8.94791e+00 neval= 182 2 OK
# vgs_drv[1] >     0.00000e+00 wc=    7.45543e-11 wcd=    7.45830e+00 neval= 127 1 OK
# vgs_drv[2] >     0.00000e+00 wc=    5.51299e-02 wcd=    1.22852e+01 neval= 149 1 OUTSIDE+
# vgs_drv[3] >     0.00000e+00 wc=    6.46699e-02 wcd=    1.95971e+01 neval= 161 1 OUTSIDE+
# vgs_drv[4] >     0.00000e+00 wc=    4.66125e-02 wcd=    1.97675e+01 neval= 146 1 OUTSIDE+
# vgs_drv[5] >     0.00000e+00 wc=    1.34344e-01 wcd=    1.41421e+01 neval= 163 2 OUTSIDE+
# vgs_drv[6] >     0.00000e+00 wc=    1.34314e-01 wcd=    1.41421e+01 neval= 163 2 OUTSIDE+
# vgs_drv[7] >     0.00000e+00 wc=    4.66716e-02 wcd=    1.00025e+01 neval= 182 2 OUTSIDE+
